@@ -51,6 +51,7 @@ class Errors(object):
         "OperationFailed": {"name": "OperationFailed", "message": "operation failed"},
         "NodeNotExists": {"name": "NodeNotExists", "message": "node not exists"},
         "AllDataNodeOffline": {"name": "AllDataNodeOffline", "message": "all data node offline"},
+        "NoUsableDataNode": {"name": "NoUsableDataNode", "message": "no usable data node"},
         "FileNotExists": {"name": "FileNotExists", "message": "file not exists"},
         "SameNameExists": {"name": "SameNameExists", "message": "same name exists"},
         "SameNameFileExists": {"name": "SameNameFileExists", "message": "same name file exists"},
@@ -169,3 +170,28 @@ def init_storage():
     for d in directories:
         if not os.path.exists(d) or not os.path.isdir(d):
             os.makedirs(d)
+
+
+def makekey(c):
+    if isinstance(c, int):
+        return c
+    elif isinstance(c, str):
+        return c.lower()
+
+
+def list_sort(l, sort_by, desc = False):
+    l_keys = []
+    l_mapping = {}
+    l_sorted = []
+    for c in l:
+        l_keys.append(c[sort_by])
+        if c[sort_by] in l_mapping:
+            l_mapping[c[sort_by]].append(c)
+        else:
+            l_mapping[c[sort_by]] = [c]
+    l_keys = list(set(l_keys))
+    l_keys.sort(key = makekey, reverse = desc)
+    for k in l_keys:
+        for c in l_mapping[k]:
+            l_sorted.append(c)
+    return l_sorted
