@@ -1,5 +1,5 @@
-function storageInit (manager_host) {
-	var $table_header = $("#remote-manager > .header-fixed > thead");
+function remoteStorageInit (manager_host) {
+    var $table_header = $("#remote-manager > .header-fixed > thead");
     var $table_header_tr = $("#remote-manager > .header-fixed > thead > tr");
     var $table_body = $("#remote-manager > .header-fixed > tbody");
     var $btn_home = $("#remote-manager #btn_home");
@@ -11,6 +11,8 @@ function storageInit (manager_host) {
     var $btn_create_ok = $("#remote-create-modal #btn-remote-create");
     var $btn_download = $("#remote-manager #btn_download");
     var $btn_download_ok = $("#remote-download-modal #btn-remote-download");
+    var $btn_preview = $("#remote-manager #btn_preview");
+    var $btn_preview_ok = $("#remote-preview-modal #btn-remote-preview");
     var $btn_cut = $("#remote-manager #btn_cut");
     var $btn_cut_ok = $("#remote-cut-modal #btn-remote-cut");
     var $btn_paste = $("#remote-manager #btn_paste");
@@ -24,8 +26,9 @@ function storageInit (manager_host) {
     var home_path = [];
     var dirs = [];
     var files = [];
+    var current_page = 1;
+    var current_page_size = 100;
 
-    var $log_console = $("textarea#log-console");
     var scrollBarSize = getBrowserScrollSize();
 
     var remote = window.location.host;
@@ -51,6 +54,8 @@ function storageInit (manager_host) {
             $btn_delete_ok.bind('click', deleteFileDir);
             $btn_download.bind('click', showDownload);
             $btn_download_ok.bind('click', downloadFileDir);
+            $btn_preview.bind('click', showPreview);
+            $btn_preview_ok.bind('click', previewFile);
             $btn_cut.bind('click', showCut);
             $btn_cut_ok.bind('click', cutFileDir);
             $btn_paste.bind('click', showPaste);
@@ -71,18 +76,17 @@ function storageInit (manager_host) {
             if (data.cmd == "init") {
                 getStorageList(data);
             } else if (data.cmd == "info") {
-                $log_console.val($log_console.val() + '\nInfo: ' + data.info);
-                $log_console.scrollTop($log_console[0].scrollHeight);
+                logConsole('Info: ' + data.info);
             } else if (data.cmd == "warning") {
-                $log_console.val($log_console.val() + '\nWarning: ' + data.info);
-                $log_console.scrollTop($log_console[0].scrollHeight);
+                logConsole('Warning: ' + data.info);
             } else if (data.cmd == "error") {
-                $log_console.val($log_console.val() + '\nError: ' + data.info);
-                $log_console.scrollTop($log_console[0].scrollHeight);
+                logConsole('Error: ' + data.info);
             } else if (data.cmd == "paste") {
                 $btn_paste.attr("disabled", false);
             } else if (data.cmd == "need_refresh") {
                 conditionRefreshDir(data.dir_path);
+            } else if (data.cmd == "preview") {
+                showPreviewContent(data.file_path, data.data, data.type);
             }
         };
 
@@ -109,56 +113,62 @@ function storageInit (manager_host) {
             "ctime",
             "mtime"
         ];
-        data.dirs.forEach(function (value, index, arrays) {
+
+        dirs = [];
+        dir_index = 0;
+        files = [];
+        file_index = 0
+        data.items.forEach(function (value, index, arrays) {
             var tr = '<tr id="table_item">';
-            for (var i=0; i<columns.length; i++) {
-                var col = columns[i];
-                if (col == 'name') {
-                    tr += '<td id="' + col + '" title="' + value[col] + '">';
-                    tr += '<div class="outer">';
-                    tr += '<div class="inner">';
-                    tr += '<span>';
-                    tr += '<input class="dir-item" name="dir" type="checkbox" id="dir_' + index + '">';
-                    tr += '</span>&nbsp;';
-                    tr += '<span class="oi oi-folder" aria-hidden="true">';
-                    tr += '</span>'
-                    tr += '<a class="dir-item" id="' + value[col] + '">&nbsp;' + value[col] + '</a>';
-                    tr += '</div>';
-                    tr += '</div>';
-                    tr += '</td>';
-                } else {
-                    tr += '<td id="' + col + '"><div class="outer"><div class="inner">&nbsp;' + value[col] + '</div></div></td>';
+            if (value.type == "Directory") {
+                for (var i=0; i<columns.length; i++) {
+                    var col = columns[i];
+                    if (col == 'name') {
+                        tr += '<td id="' + col + '" title="' + value[col] + '">';
+                        tr += '<div class="outer">';
+                        tr += '<div class="inner">';
+                        tr += '<span>';
+                        tr += '<input class="dir-item" name="dir" type="checkbox" id="dir_' + dir_index + '">';
+                        tr += '</span>&nbsp;';
+                        tr += '<span class="oi oi-folder" aria-hidden="true">';
+                        tr += '</span>'
+                        tr += '<a class="dir-item" id="' + value[col] + '">&nbsp;' + value[col] + '</a>';
+                        tr += '</div>';
+                        tr += '</div>';
+                        tr += '</td>';
+                    } else {
+                        tr += '<td id="' + col + '"><div class="outer"><div class="inner">&nbsp;' + value[col] + '</div></div></td>';
+                    }
                 }
+                dirs.push(value);
+                dir_index += 1;
+            } else {
+                for (var i=0; i<columns.length; i++) {
+                    var col = columns[i];
+                    if (col == 'name') {
+                        tr += '<td id="' + col + '" title="' + value[col] + '">';
+                        tr += '<div class="outer">';
+                        tr += '<div class="inner">';
+                        tr += '<span>';
+                        tr += '<input class="file-item" name="file" type="checkbox" id="file_' + file_index + '">';
+                        tr += '</span>&nbsp;';
+                        tr += '<span class="oi oi-file" aria-hidden="true">';
+                        tr += '</span>'
+                        tr += '<a class="file-item" id="file_' + file_index + '">&nbsp;' + value[col] + '</a>';
+                        tr += '</div>';
+                        tr += '</div>';
+                        tr += '</td>';
+                    } else {
+                        tr += '<td id="' + col + '"><div class="outer"><div class="inner">&nbsp;' + value[col] + '</div></div></td>';
+                    }
+                }
+                files.push(value);
+                file_index += 1;
             }
+            
             tr += '</tr>';
             $table_body.append(tr);
         });
-        dirs = data.dirs;
-        data.files.forEach(function (value, index, arrays) {
-            var tr = '<tr id="table_item">';
-            for (var i=0; i<columns.length; i++) {
-                var col = columns[i];
-                if (col == 'name') {
-                    tr += '<td id="' + col + '" title="' + value[col] + '">';
-                    tr += '<div class="outer">';
-                    tr += '<div class="inner">';
-                    tr += '<span>';
-                    tr += '<input class="file-item" name="file" type="checkbox" id="file_' + index + '">';
-                    tr += '</span>&nbsp;';
-                    tr += '<span class="oi oi-file" aria-hidden="true">';
-                    tr += '</span>'
-                    tr += '<a class="file-item" id="file_' + index + '">&nbsp;' + value[col] + '</a>';
-                    tr += '</div>';
-                    tr += '</div>';
-                    tr += '</td>';
-                } else {
-                    tr += '<td id="' + col + '"><div class="outer"><div class="inner">&nbsp;' + value[col] + '</div></div></td>';
-                }
-            }
-            tr += '</tr>';
-            $table_body.append(tr);
-        });
-        files = data.files;
 
         dir_path = data.dir_path;
         home_path = data.home_path;
@@ -175,6 +185,11 @@ function storageInit (manager_host) {
             $table_header.css({"margin-right": 0});
         }
 
+        generatePagination('#remote-manager #ul-pagination', current_page, current_page_size, 5, data.total);
+        $('#remote-manager a.page-num').bind('click', changePage);
+        $('#remote-manager a.previous-page').bind('click', previousPage);
+        $('#remote-manager a.next-page').bind('click', nextPage);
+
         addColumnsCSS(columns);
         $("a.dir-item").css("cursor", "pointer");
         $("a.file-item").css("cursor", "pointer");
@@ -183,13 +198,17 @@ function storageInit (manager_host) {
     }
 
     function goHomeDir() {
+        current_page = 1;
         var data = {};
         data.cmd = "cd";
         data.dir_path = home_path;
+        data.offset = (current_page - 1) * current_page_size;
+        data.limit = current_page_size;
         socket.send(JSON.stringify(data));
     }
 
     function goParentDir() {
+        current_page = 1;
         var index = dir_path.length - 1;
         var data = {};
         data.cmd = "cd";
@@ -197,6 +216,8 @@ function storageInit (manager_host) {
             index = 1;
         }
         data.dir_path = dir_path.slice(0, index);
+        data.offset = (current_page - 1) * current_page_size;
+        data.limit = current_page_size;
         socket.send(JSON.stringify(data));
     }
 
@@ -204,6 +225,8 @@ function storageInit (manager_host) {
         var data = {};
         data.cmd = "refresh";
         data.dir_path = dir_path;
+        data.offset = (current_page - 1) * current_page_size;
+        data.limit = current_page_size;
         socket.send(JSON.stringify(data));
     }
 
@@ -215,11 +238,14 @@ function storageInit (manager_host) {
     }
 
     function openDir(event) {
+        current_page = 1;
         var dir_name = $(this).attr("id");
         var data = {};
         data.cmd = "cd";
         dir_path.push(dir_name);
         data.dir_path = dir_path;
+        data.offset = (current_page - 1) * current_page_size;
+        data.limit = current_page_size;
         socket.send(JSON.stringify(data));
         event.stopPropagation()
     }
@@ -235,6 +261,8 @@ function storageInit (manager_host) {
         data.cmd = "mkdir";
         data.name = name;
         data.dir_path = dir_path;
+        data.offset = (current_page - 1) * current_page_size;
+        data.limit = current_page_size;
         socket.send(JSON.stringify(data));
     }
 
@@ -267,6 +295,8 @@ function storageInit (manager_host) {
         data.old_name = old_name;
         data.new_name = new_name;
         data.dir_path = dir_path;
+        data.offset = (current_page - 1) * current_page_size;
+        data.limit = current_page_size;
         socket.send(JSON.stringify(data));
     }
 
@@ -292,6 +322,12 @@ function storageInit (manager_host) {
         data.files = delete_files;
         data.dir_path = dir_path;
         socket.send(JSON.stringify(data));
+        delete_dirs.forEach(function (value, index, arrays) {
+            logConsole("Info: Deleting remote directory [" + namePathJoin(dir_path, value.name) + "] ...");
+        });
+        delete_files.forEach(function (value, index, arrays) {
+            logConsole("Info: Deleting remote file [" + namePathJoin(dir_path, value.name) + "] ...");
+        });
     }
 
     function showDownload() {
@@ -317,6 +353,37 @@ function storageInit (manager_host) {
         data.remote_path = $('#remote-manager input.dir-path').val();;
         data.local_path = $('#local-manager input.dir-path').val();
         socket.send(JSON.stringify(data));
+        download_dirs.forEach(function (value, index, arrays) {
+            logConsole("Info: Downloading directory [" + namePathJoin(dir_path, value.name) + "] to [" + data.local_path + "] ...");
+        });
+        download_files.forEach(function (value, index, arrays) {
+            logConsole("Info: Downloading file [" + namePathJoin(dir_path, value.name) + "] to [" + data.local_path + "] ...");
+        });
+    }
+
+    function showPreview() {
+        $('#remote-preview-modal').modal('show');
+    }
+
+    function previewFile() {
+        $('#remote-preview-modal').modal('hide');
+        var num = Number($("#remote-manager input[type=checkbox]:checked").attr("id").split("_")[1]);
+        var data = {};
+        data.cmd = "preview";
+        data.file = files[num];
+        data.dir_path = dir_path;
+        socket.send(JSON.stringify(data));
+        logConsole("Info: Loading file [" + namePathJoin(dir_path, data.file.name) + "] preview info ...");
+    }
+
+    function showPreviewContent(file_path, data, file_type) {
+        if (file_type == ".zip") {
+            document.getElementById("file-preview-json").textContent = JSON.stringify(data, undefined, 4);
+        } else {
+            document.getElementById("file-preview-json").textContent = data;
+        }
+        $('#remote-preview-file-modal').modal('show');
+        logConsole("Info: Load file [" + file_path + "] preview info success");
     }
 
     function showCut() {
@@ -353,6 +420,7 @@ function storageInit (manager_host) {
         data.cmd = "paste";
         data.dir_path = dir_path;
         socket.send(JSON.stringify(data));
+        logConsole("Info: Pasting remote files & directories ...");
     }
 
     function showFileDetail() {
@@ -397,6 +465,45 @@ function storageInit (manager_host) {
         data.replica = Number($('#remote-update-modal input#update-replica').val());
         socket.send(JSON.stringify(data));
         $('#remote-update-modal').modal('hide');
+        update_dirs.forEach(function (value, index, arrays) {
+            logConsole("Info: Updating remote directory [" + namePathJoin(dir_path, value.name) + "] ...");
+        });
+        update_files.forEach(function (value, index, arrays) {
+            logConsole("Info: Updating remote file [" + namePathJoin(dir_path, value.name) + "] ...");
+        });
+    }
+
+    function changePage() {
+        current_page = Number($(this)[0].innerText);
+        var data = {};
+        data.cmd = "change_page";
+        data.dir_path = dir_path;
+        data.offset = (current_page - 1) * current_page_size;
+        data.limit = current_page_size;
+        socket.send(JSON.stringify(data));
+    }
+
+    function previousPage() {
+        current_page--;
+        if (current_page < 1) {
+            current_page = 1;
+        }
+        var data = {};
+        data.cmd = "change_page";
+        data.dir_path = dir_path;
+        data.offset = (current_page - 1) * current_page_size;
+        data.limit = current_page_size;
+        socket.send(JSON.stringify(data));
+    }
+
+    function nextPage() {
+        current_page++;
+        var data = {};
+        data.cmd = "change_page";
+        data.dir_path = dir_path;
+        data.offset = (current_page - 1) * current_page_size;
+        data.limit = current_page_size;
+        socket.send(JSON.stringify(data));
     }
 
     function inputSelect(event) {
@@ -422,6 +529,7 @@ function storageInit (manager_host) {
             $btn_rename.attr("disabled", false);
             $btn_create.attr("disabled", true);
             $btn_download.attr("disabled", false);
+            $btn_preview.attr("disabled", false);
             // $btn_paste.attr("disabled", false);
             $btn_update.attr("disabled", false);
             $btn_cut.attr("disabled", false);
@@ -432,6 +540,7 @@ function storageInit (manager_host) {
             $btn_rename.attr("disabled", true);
             $btn_create.attr("disabled", true);
             $btn_download.attr("disabled", false);
+            $btn_preview.attr("disabled", true);
             // $btn_paste.attr("disabled", false);
             $btn_update.attr("disabled", false);
             $btn_cut.attr("disabled", false);
@@ -441,6 +550,7 @@ function storageInit (manager_host) {
             $btn_rename.attr("disabled", false);
             $btn_create.attr("disabled", true);
             $btn_download.attr("disabled", false);
+            $btn_preview.attr("disabled", true);
             // $btn_paste.attr("disabled", false);
             $btn_update.attr("disabled", false);
             $btn_cut.attr("disabled", false);
@@ -450,6 +560,7 @@ function storageInit (manager_host) {
             $btn_rename.attr("disabled", true);
             $btn_create.attr("disabled", false);
             $btn_download.attr("disabled", true);
+            $btn_preview.attr("disabled", true);
             // $btn_paste.attr("disabled", false);
             $btn_update.attr("disabled", true);
             $btn_cut.attr("disabled", true);
@@ -459,6 +570,7 @@ function storageInit (manager_host) {
             $btn_rename.attr("disabled", true);
             $btn_create.attr("disabled", true);
             $btn_download.attr("disabled", false);
+            $btn_preview.attr("disabled", true);
             // $btn_paste.attr("disabled", false);
             $btn_update.attr("disabled", false);
             $btn_cut.attr("disabled", false);
@@ -472,7 +584,7 @@ function storageInit (manager_host) {
     function resetModal(e) {
         $("#" + e.target.id).find("input:text").val("");
         $("#" + e.target.id).find("input:file").val(null);
-        $("#" + e.target.id).$('input#update-replica').val(1);
+        $("#" + e.target.id).find('input#update-replica').val(1);
         $("#" + e.target.id).find(".custom-file-label").html("Choose file");
         $("#" + e.target.id).find("textarea").val("");
     }
